/**
 * Same as the default `Error`, but it is JSON serializable.
 * @source https://iaincollins.medium.com/error-handling-in-javascript-a6172ccdf9af
 */
export class UnknownError extends Error {
    constructor(error) {
        var _a;
        // Support passing error or string
        super((_a = error === null || error === void 0 ? void 0 : error.message) !== null && _a !== void 0 ? _a : error);
        this.name = "UnknownError";
        if (error instanceof Error) {
            this.stack = error.stack;
        }
    }
    toJSON() {
        return {
            name: this.name,
            message: this.message,
            stack: this.stack,
        };
    }
}
export class OAuthCallbackError extends UnknownError {
    constructor() {
        super(...arguments);
        this.name = "OAuthCallbackError";
    }
}
/**
 * Thrown when an Email address is already associated with an account
 * but the user is trying an OAuth account that is not linked to it.
 */
export class AccountNotLinkedError extends UnknownError {
    constructor() {
        super(...arguments);
        this.name = "AccountNotLinkedError";
    }
}
export function upperSnake(s) {
    return s.replace(/([A-Z])/g, "_$1").toUpperCase();
}
export function capitalize(s) {
    return `${s[0].toUpperCase()}${s.slice(1)}`;
}
/**
 * Wraps an object of methods and adds error handling.
 * @param {import("types").EventCallbacks} methods
 * @param {import("types").LoggerInstance} logger
 * @return {import("types").EventCallbacks}
 */
export function eventsErrorHandler(methods, logger) {
    return Object.entries(methods).reduce((acc, [name, method]) => {
        acc[name] = async (...args) => {
            try {
                return await method(...args);
            }
            catch (e) {
                logger.error(`${upperSnake(name)}_EVENT_ERROR`, e);
            }
        };
        return acc;
    }, {});
}
/**
 * Handles adapter induced errors.
 * @param {import("types/adapters").Adapter} [adapter]
 * @param {import("types").LoggerInstance} logger
 * @return {import("types/adapters").Adapter}
 */
export function adapterErrorHandler(adapter, logger) {
    if (!adapter)
        return;
    return Object.keys(adapter).reduce((acc, method) => {
        acc[method] = async (...args) => {
            try {
                logger.debug(`adapter_${method}`, ...args);
                const adapterMethod = adapter[method];
                return await adapterMethod(...args);
            }
            catch (error) {
                logger.error(`adapter_error_${method}`, error);
                const e = new UnknownError(error);
                e.name = `${capitalize(method)}Error`;
                throw e;
            }
        };
        return acc;
    }, {});
}
