import { openidClient } from "../oauth/client";
import { oAuth1Client } from "../oauth/client-legacy";
import { createState } from "../oauth/state-handler";
import { createPKCE } from "../oauth/pkce-handler";
/**
 *
 * Generates an authorization/request token URL.
 *
 * [OAuth 2](https://www.oauth.com/oauth2-servers/authorization/the-authorization-request/) | [OAuth 1](https://oauth.net/core/1.0a/#auth_step2)
 * @type {import("types/internals").NextAuthApiHandler}
 * @returns {string}
 */
export default async function getAuthorizationUrl(req, res) {
    var _a, _b;
    const { logger } = req.options;
    try {
        /** @type {import("types/providers").OAuthConfig} */
        const provider = req.options.provider;
        let params = {};
        if (typeof provider.authorization === "string") {
            const parsedUrl = new URL(provider.authorization);
            const parsedParams = Object.fromEntries(parsedUrl.searchParams.entries());
            params = { ...params, ...parsedParams };
        }
        else {
            params = { ...params, ...(_a = provider.authorization) === null || _a === void 0 ? void 0 : _a.params };
        }
        params = { ...params, ...req.query };
        // Handle OAuth v1.x
        if ((_b = provider.version) === null || _b === void 0 ? void 0 : _b.startsWith("1.")) {
            const client = oAuth1Client(req.options);
            const tokens = await client.getOAuthRequestToken(params);
            const url = `${provider.authorization}?${new URLSearchParams({
                oauth_token: tokens.oauth_token,
                oauth_token_secret: tokens.oauth_token_secret,
                ...tokens.params,
            })}`;
            logger.debug("GET_AUTHORIZATION_URL", { url });
            return url;
        }
        const client = await openidClient(req.options);
        const pkce = await createPKCE(req, res);
        const url = client.authorizationUrl({
            ...params,
            ...pkce,
            state: createState(req),
        });
        logger.debug("GET_AUTHORIZATION_URL", { url });
        return url;
    }
    catch (error) {
        logger.error("GET_AUTHORIZATION_URL_ERROR", error);
        throw error;
    }
}
